#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::Webservice_BasketService
# base         DE_EPAGES::Core::API::Test;
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Webservice::BasketService
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::Webservice_BasketService;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use strict;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::PayPalExpress::API::Webservice::BasketService;
use DE_EPAGES::Test::API::TestTools qw (TestExistsMethod);
use DE_EPAGES::Order::API::Object::Basket;

sub test {
    my $self = shift;

    $self->test_importBasket_negative_callWithoutObject();
    $self->test_importBasket_negative_callWithNonBasketObject();
    $self->test_importBasket_negative_callWithoutDefinedContainer();
    $self->test_importBasket_positive_with_completeBasket();

    $self->test_AllSimpleGetters();

    $self->test_wasTriggeredWithShortcutButton();
    $self->test_wasTriggeredWithShortcutButton_with_classicOrMultistepCheckout();

    $self->test_isMultistepCheckout_positive();
    $self->test_isMultistepCheckout_negative();

    $self->test_refresh();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_importBasket_negative_callWithoutObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $Basket = undef;
    my $WasImportSucessful = $BasketService->importBasket($Basket);

    is($WasImportSucessful, 0 ,"$SubTestName - tests if the basket import was not sucessful if there is no basket");

}
#----------------------------------------------------------------------------------------
sub test_importBasket_negative_callWithNonBasketObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $NonBasketObject = $self->_createFaultyObject();
    my $WasImportSucessful = $BasketService->importBasket($NonBasketObject);

    is($WasImportSucessful, 0 ,"$SubTestName - tests if the basket import was not sucessful if there is a non basket object");

}
#----------------------------------------------------------------------------------------
sub test_importBasket_negative_callWithoutDefinedContainer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasketWithOutContainer();

    my $WasImportSucessful = $BasketService->importBasket($Basket);

    is($WasImportSucessful, 0 ,"$SubTestName - tests if the basket import was not sucessful if there is basket object without container");

}
#----------------------------------------------------------------------------------------
sub test_importBasket_positive_with_completeBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasket();
    my $WasImportSucessful = $BasketService->importBasket($Basket);

    ok($WasImportSucessful, "$SubTestName - tests if the basket import was sucessful");

}
#----------------------------------------------------------------------------------------
sub test_AllSimpleGetters {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasket();
    my $WasImportSucessful = $BasketService->importBasket($Basket);

    ok($WasImportSucessful, "$SubTestName - tests if the basket import was sucessful");

    my $ExpectedBasketID = 123456; ## no critic ( ProhibitMagicNumbers )
    is($BasketService->getBasketID(), $ExpectedBasketID, "$SubTestName - test if we get the basket ID");
    isa_ok($BasketService->getLineItemPayment(),
        'DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress',
        "$SubTestName - test if  we get the LineItemPaymentPayPalExpress object"
    );
    isa_ok($BasketService->getBasket(),
        'DE_EPAGES::Order::API::Object::Basket',
        "$SubTestName - test if  we get the basket object"
    );

}
#----------------------------------------------------------------------------------------
sub test_wasTriggeredWithShortcutButton {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasket();

    my $WasImportSucessful = $BasketService->importBasket($Basket);
    my $IsShortcutCheckout = $BasketService->wasTriggeredWithShortcutButton();

    ok($WasImportSucessful, "$SubTestName - tests if the basket import was sucessful");
    ok($IsShortcutCheckout,"$SubTestName - tests if the shortcut-button switch is working for paypalexpressCheckout");

}
#----------------------------------------------------------------------------------------
sub test_wasTriggeredWithShortcutButton_with_classicOrMultistepCheckout {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $IsMultistepCheckout = 0;
    my $Basket = $self->_buildNormalCheckoutBasket($IsMultistepCheckout);

    my $WasImportSuccessful = $BasketService->importBasket($Basket);
    my $IsButtonCheckout = $BasketService->wasTriggeredWithShortcutButton();

    ok( $WasImportSuccessful, "$SubTestName - tests if the basket import was sucessful" );
    is( $IsButtonCheckout, 0, "$SubTestName - tests if this button switch is working for normal checkout" );

}
#----------------------------------------------------------------------------------------
sub test_isMultistepCheckout_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $IsMultistepCheckout = 1;
    my $Basket = $self->_buildNormalCheckoutBasket($IsMultistepCheckout);

    my $WasImportSucessful = $BasketService->importBasket($Basket);
    my $Result = $BasketService->isMultistepCheckout();

    ok($WasImportSucessful, "$SubTestName - tests if the basket import was sucessful");
    ok($Result, "$SubTestName - tests if positive is Multistep checkout");

}
#----------------------------------------------------------------------------------------
sub test_isMultistepCheckout_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $IsMultistepCheckout = 0;
    my $Basket = $self->_buildNormalCheckoutBasket($IsMultistepCheckout);

    my $WasImportSucessful = $BasketService->importBasket($Basket);
    my $Result = $BasketService->isMultistepCheckout();

    ok($WasImportSucessful, "$SubTestName - tests if the basket import was sucessful");
    is($Result, 0, "$SubTestName - tests if the check returns false, if this is not a Multistep checkout");

}
#----------------------------------------------------------------------------------------
sub test_refresh {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $BasketService = $self->_createBasketService();
    my $Basket = $self->_buildBasket();
    my $WasImportSucessful = $BasketService->importBasket($Basket);

    $BasketService->refresh();

    ok($WasImportSucessful, "$SubTestName - tests if the basket import was successful");

    my $LineItemPayment = $BasketService->getLineItemPayment();

    my $ExpectedAmount = 100; ## no critic (ProhibitMagicNumbers)
    is($LineItemPayment->get('Amount'), $ExpectedAmount, "$SubTestName - test if the Order amount is set correctly in the payment methods");
    is($LineItemPayment->get('ShippingAddressChanged'), 0, "$SubTestName - test if ShippingAddressChanged is set");
    my $ExpectedObjectID = 1234; ## no critic (ProhibitMagicNumbers)
    is($LineItemPayment->get('ShippingAddressID'), $ExpectedObjectID, "$SubTestName - test if ShippingAddressID is set");

    is($Basket->get('IsAddressOK'), 1, "$SubTestName - test if IsAddressOK is set");
    is($Basket->{'__updateTaxArea_CallCounter'}, 1, "$SubTestName - test if updateTaxArea was called");
    is($Basket->{'__updateTaxAreaDigital_CallCounter'}, 1, "$SubTestName - test if updateTaxAreaDigital was called one time");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _createBasketService {
    my $self = shift;
    my ($Order) = @_;
    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Webservice::BasketService',
                   $aParameterList
              );

    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_UpdateTaxModel',
        undef,
        [{'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer'}]
    );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _buildBasket {
    my $self = shift;
    my $IsExpressCheckout = 1;
    my $LineItemPaymentPayPalExpress = $self->_createLineItemPaymentPayPalExpress($IsExpressCheckout);
    my $hAttributes = {
        'ID' => 123456,
        'Alias' => 'MyTestAddress',
        'instanceOf'=>'Basket',
        'LineItemContainer' => $self->_createLineItemContainer($LineItemPaymentPayPalExpress),
        'ShippingAddress'=>$self->_createShippingAddress(),
    };
    return $self->_createBasket($hAttributes);
}
#----------------------------------------------------------------------------------------
sub _buildNormalCheckoutBasket {
    my $self = shift;
    my ($IsMultistepCheckout) = @_;
    my $IsExpressCheckout = 0;
    my $LineItemPaymentPayPalExpress = $self->_createLineItemPaymentPayPalExpress($IsExpressCheckout);
    my $hAttributes = {
        'instanceOf'=>'Basket',
        'LineItemContainer' => $self->_createLineItemContainer($LineItemPaymentPayPalExpress),
        'Site' => $self->_createShop($IsMultistepCheckout),
    };
    return $self->_createBasket($hAttributes);
}
#----------------------------------------------------------------------------------------
sub _buildBasketWithOutContainer {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'MyTestAddress',
        'instanceOf'=>'Basket',
        'LineItemContainer' => undef,
    };
    return $self->_createBasket($hAttributes);
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($hAttributes) = @_;
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hAttributes
    );
    TestExistsMethod('DE_EPAGES::Order::API::Object::Basket','updateTaxArea');
    $Basket->mock('updateTaxArea', sub{
            my $Mockedself = shift;
            $Mockedself->{'__updateTaxArea_CallCounter'} += 1;
        }
    );
    TestExistsMethod('DE_EPAGES::Order::API::Object::Basket','updateTaxAreaDigital');
    $Basket->mock('updateTaxAreaDigital', sub{
            my $Mockedself = shift;
            $Mockedself->{'__updateTaxAreaDigital_CallCounter'} += 1;
        }
    );
    return $Basket;
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;
    my ($LineItemPaymentPayPalExpress) = @_;

    my $hAttributes = {
        'Alias' => 'MyTestAddress',
        'Payment' => $LineItemPaymentPayPalExpress,
        'GrandTotal' => '100',
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hAttributes
    );

    return $LineItemContainer;
}
#----------------------------------------------------------------------------------------
sub _createLineItemPaymentPayPalExpress {
    my $self = shift;
    my ($IsExpressCheckout) = @_;

    my $hAttributes = {
        'Alias' => 'MyTestAddress',
        'instanceOf'=>'LineItemPaymentPayPalExpress',
        'IsExpressCheckout' => $IsExpressCheckout,
        'ShippingAddressID' => 123456987,
        'ShippingAddressChanged' => 1,
        'Amount' => 0,
    };
    my $LineItemPaymentPayPalExpress = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress',
        $hAttributes
    );

    return $LineItemPaymentPayPalExpress;
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($IsMultistepCheckout) = @_;
    my $hAttributes = {
        'OrderProcess' => $IsMultistepCheckout,
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );

    return $Shop;
}
#----------------------------------------------------------------------------------------
sub _createShippingAddress {
    my $self = shift;

    my $hAttributes = {
        'ID' => 1234,
        'FirstName' => 'DefaultFirstName',
        'LastName' => 'DefaultLastName',
    };
    my $ShippingAddress = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hAttributes
    );

    return $ShippingAddress;
}
#----------------------------------------------------------------------------------------
sub _createFaultyObject {
    my $self = shift;

    my $hAttributes = {
        'instanceOf' => 'FaultyObject'
    };
    my $Object = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Object', $hAttributes );

    return $Object;
}
__PACKAGE__->runScript();