#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::Webservice_Response
# base         base DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit tests for Response
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::Webservice_Response;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalExpress::API::Webservice::Response;
use Test::More;
use strict;

sub test {
    my $self = shift;
    $self->test_getBillingAddress();
    $self->test_getShippingAddress();
    $self->test_EmptyRequest();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_getBillingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Response = $self->_buildWebserviceResponseWithValidBillingAddress();

    my $BillingAddressData = $Response->getBillingAddressData();
    my $ExpectedBillingAddressData = {
        'Phone' => undef,
        'FirstName' => 'aFirstName',
        'LastName' => undef,
        'CountryID' => undef,
        'MiddleName' => undef,
        'Salutation' => undef,
        'EMail' => 'ich@du.wir',
        'Company' => undef
    };
    is_deeply(
        $BillingAddressData,
        $ExpectedBillingAddressData,
        "$SubTestName - tests if we get the billing data from the request in epages data structure"
    );

    return;
}
#----------------------------------------------------------------------------------------
sub test_getShippingAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Response = $self->_buildWebserviceResponseWithValidShippingAddress();

    my $AddressData = $Response->getShippingAddressData();

    my $ExpectedAddressData = {
        'Street2' => undef,
        'Zipcode' => undef,
        'LastName' => 'FirstName Lastname',
        'Street' => undef,
        'CountryID' => undef,
        'City' => 'Paris',
        'State' => undef
    };
    is_deeply(
        $AddressData,
        $ExpectedAddressData,
        "$SubTestName - tests if we get the shipping data from the request in epages data structure"
    );

    return;
}
#----------------------------------------------------------------------------------------
sub test_EmptyRequest {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Response = $self->_buildWebserviceResponseWithEmptyRequest();

    my $hShippingAddressData = $Response->getShippingAddressData();
    my $hBillingAddressData = $Response->getBillingAddressData();

    is($hShippingAddressData, undef, "$SubTestName - tests if we get undef for Shipping Address if the response is empty");
    is($hBillingAddressData, undef, "$SubTestName - tests if we get undef for Billing Address if the response is empty");

    return;
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildWebserviceResponseWithValidBillingAddress {
    my $self = shift;
    my $WebserviceResponse = {};
    $WebserviceResponse->{'EMAIL'} = 'ich@du.wir',
    $WebserviceResponse->{'FIRSTNAME'} = 'aFirstName',
    return $self->_createWebserviceResponse($WebserviceResponse);
}
#----------------------------------------------------------------------------------------
sub _buildWebserviceResponseWithValidShippingAddress {
    my $self = shift;
    my $WebserviceResponse = {};
    $WebserviceResponse->{'PAYMENTREQUEST_0_SHIPTOCITY'} = 'Paris',
    $WebserviceResponse->{'PAYMENTREQUEST_0_SHIPTONAME'} = 'FirstName Lastname',
    return $self->_createWebserviceResponse($WebserviceResponse);
}
#----------------------------------------------------------------------------------------
sub _buildWebserviceResponseWithEmptyRequest {
    my $self = shift;
    return $self->_createWebserviceResponse();
}
#----------------------------------------------------------------------------------------
sub _createWebserviceResponse {
    my $self = shift;
    my ($hResponseFromPayPal) = @_;
    my $Response = DE_EPAGES::PayPalExpress::API::Webservice::Response->new($hResponseFromPayPal);
    return $Response;
}
#----------------------------------------------------------------------------------------
__PACKAGE__->runScript();